function FK = FUN_NLFK(par, NLFKgeom, states, t)
% FUN_NLFK_st_dy computes nonlinearn Froude-Krylov forces, with respect to the Reference point
% and expressed in the body-fixed frame (with the origin in the Reference point)
%
%   INPUT:
%->       par      = structure containing solme simulation parameters
%->       NLFKgeom = file containing all geom characteristics for the NLFK computation
%->       states   = [xR, yR, zR, PHI, THETA, PSI] = 6-degree of freedom displacements of the RefP in the WF
%->       t        = time instant for the etaFun evaluation
%
%   OUTPuT:
%->       FK      = Total (static and dynamic) nonlinear Froude-Krylov force
%
%
%
%
% Function written by Dr. Giuseppe Giorgi (ORCID: 0000-0002-1649-9438) under the H2020-MCSA-IF-2018 fellowship
% Grant agreement: 832140
% NLFK4ALL - OpenWave: Validation and Optimization of an Open-Source Novel Nonlinear Froude-Krylov Model for Advanced Design of Wave Energy Converters
% Last update: 09/11/2019

FK = zeros(7,1);

%%
% WF is world (fixed and inertial) frame
% BF is body-fixed (and non-inertial) frame

xR    = states(1);               % x displacement of the RefP, in the WF
yR    = states(2);               % y displacement of the RefP, in the WF
zR    = states(3);               % z displacement of the RefP, in the WF
PHI   = states(4);               % rotation about an axis parallel to the x-axis, passing through the RefP
THETA = states(5);               % rotation about an axis parallel to the y-axis, passing through the RefP
PSI   = states(6);               % rotation about an axis parallel to the z-axis, passing through the RefP
%%
CoG_BF = [0, 0, NLFKgeom.zCoG - NLFKgeom.zRefP]'; % Position of the CoG in the BF

refP   = [xR, yR, zR + NLFKgeom.zRefP]'; % Position of the refP, after displacement, in the WF coordinates

Rx = [1 0         0        ; 
      0 cos(PHI) -sin(PHI) ; 
      0 sin(PHI)  cos(PHI)];         % rotation matrix around x
  
Ry = [cos(THETA) 0 sin(THETA) ; 
      0          1 0          ; 
     -sin(THETA) 0 cos(THETA)]; % rotation matrix around y
 
Rz = [cos(PSI) -sin(PSI) 0 ;
      sin(PSI)  cos(PSI) 0 ; 
      0         0        1];         % rotation matrix around z

% The order of rotation is important! There are different conventions...
% Marine applications, [Fossen, Handbook of marine craft hydrodynamics and motion control, 2011]

RotPure = Rz*Ry*Rx;                   % Pure rotation    matrix in 3D: XYZ  = [x;y;z]  , such that WF = Rot*BF
R       = [RotPure refP; 0 0 0 1];  % Roto-translation matrix in 4D: XYZ1 = [x;y;z;1], such that WF = R*BF
 
%%
Fg    = [0 0 -par.g*par.Mass]'; % Gravity force, vector (3x1)

%% Hollow floater 

for iPatch = NLFKgeom.Patches.cylCoor
    
    name = {['Patch_',num2str(iPatch)]};
    fun  = NLFKgeom.(name{1}).fun;    
    rho1 = NLFKgeom.(name{1}).rho1;
    
    if ismember(iPatch,NLFKgeom.Patches.freeSurf)
        flatLimit = par.etaFun(xR, t);
        rho2      = FUN_findIntersection(par.modIntersect,par, NLFKgeom, states, t, R, iPatch, flatLimit);
    else
        rho2 = NLFKgeom.(name{1}).rho2;
    end
    
    % (x,z) coordinates of the lateral surface, in the WF, used to compute the pressure
    xLat = @(th,rho) fun(rho).*(R(1,1)*cos(th) + R(1,2).*sin(th)) + R(1,3)*rho + R(1,4); 
    zLat = @(th,rho) fun(rho).*(R(3,1)*cos(th) + R(3,2).*sin(th)) + R(3,3)*rho + R(3,4);

    % Components of (e_rho x e_th) and  (pos x (e_rho x e_th))
    n1Lat = NLFKgeom.(name{1}).n1;
    n2Lat = NLFKgeom.(name{1}).n2;
    n3Lat = NLFKgeom.(name{1}).n3;
    n4Lat = NLFKgeom.(name{1}).n4;
    n5Lat = NLFKgeom.(name{1}).n5;

    p = @(th,rho) -par.gamma*zLat(th,rho) + par.pDy(xLat(th,rho), zLat(th,rho), t, xR); % pressure for the cylindrical coordinates
                
    FK(1) = FK(1) + integral2(@(th,rho) p (th,rho).*n1Lat (th,rho),-pi,pi,rho1,rho2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);
    FK(2) = FK(2) + integral2(@(th,rho) p (th,rho).*n2Lat (th,rho),-pi,pi,rho1,rho2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);
    FK(3) = FK(3) + integral2(@(th,rho) p (th,rho).*n3Lat (th,rho),-pi,pi,rho1,rho2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);
    FK(4) = FK(4) + integral2(@(th,rho) p (th,rho).*n4Lat (th,rho),-pi,pi,rho1,rho2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);
    FK(5) = FK(5) + integral2(@(th,rho) p (th,rho).*n5Lat (th,rho),-pi,pi,rho1,rho2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);

    
end

%%

for iPatch = NLFKgeom.Patches.polarCoor
    name = {['Patch_',num2str(iPatch)]};
    z0   = NLFKgeom.(name{1}).z0;
    
    % (x,z) coordinates of the disk, in the WF, used to compute the pressure
    xDisk = @(th,r) r.*(R(1,1)*cos(th) + R(1,2).*sin(th)) + R(1,3)*z0 +R(1,4); 
    zDisk = @(th,r) r.*(R(3,1)*cos(th) + R(3,2).*sin(th)) + R(3,3)*z0 +R(3,4); 

    % Components of (e_r x e_th) and (pos x (e_r x e_th)). Other components are zero
    n3Disk = NLFKgeom.(name{1}).n3;
    n4Disk = NLFKgeom.(name{1}).n4;
    n5Disk = NLFKgeom.(name{1}).n5;

    p = @(th,r)   -par.gamma*zDisk(th,r) + par.pDy(xDisk(th,r), zDisk(th,r), t, xR); % pressure for the disk

    r1 = NLFKgeom.(name{1}).r1;
    r2 = NLFKgeom.(name{1}).r2;

    FK(3) = FK(3) + integral2(@(th,r)   p(th,r)  .*n3Disk(th,r)  ,-pi,pi,r1,r2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);
    FK(4) = FK(4) + integral2(@(th,r)   p(th,r)  .*n4Disk(th,r)  ,-pi,pi,r1,r2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);
    FK(5) = FK(5) + integral2(@(th,r)   p(th,r)  .*n5Disk(th,r)  ,-pi,pi,r1,r2,'RelTol',par.RelTol,'AbsTol',par.AbsTol);

end



FK(1:3) = FK(1:3) + RotPure'*Fg ; % Rotating the gravity force in the body frame, and summing it to the integral of the static pressure
FK(4:6) = FK(4:6) + cross(CoG_BF, RotPure'*Fg);  % Adding the contribution of the gravity force at the centre of gravity to the total torque

end










%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function rho2 = FUN_findIntersection(modIntersect, par, NLFKgeom, states, t, R, iPatch, flatLimit)
% FUN_findIntersection finds finds the interstection, in the body-fixed frame between the free surface elevation and one patch
%
% INPUT:
%->     modIntersect: can be 'exact', 'linear', 'flat'
%->     par         : structure containing the function for the surface elevation -> par.etaFun
%->     NLFKgeom    : structuer with geometrical charachteristics of the floater
%->     states      : displacements of the floater, in the world-frame
%->     t           : time instanct
%->     R           : 4D rotation matrix
%->     iPatch      : patch index
%->     flatLimit   : horizontal intersection, in the WF, used if modIntersect = 'flat'
%
% OUTPUT:
%->     rho2: function of theta that returns the parametric intersection in BF
%

RefP  = [states(1), states(2), states(3) + NLFKgeom.zRefP]'; % Position of the RefP, in the WF
b2w  = @(XYZ1) R*XYZ1;                                     % Function Body-frame(XYZ1) to World-frame(XYZ1)
name = {['Patch_',num2str(iPatch)]};
fun  = NLFKgeom.(name{1}).fun;   
switch modIntersect
    case 'exact'
        BF   = @(th,rho) [fun(rho).*cos(th);  fun(rho).*sin(th); rho; ones(size(th))];     % function from (th,rho) to XYZ1_bf 
        X    = @(th,rho) [1 0 0 0]*b2w(BF(th,rho));                                        % function from (th,rho) to just x in the world frame
        etaX = @(th,rho) par.etaFun(X(th,rho), t);                                         % function from (th,rho) to eta in the world frame

        % The zeros of Fun_inter define the exact intersection between body and wave
        Fun_inter = @(th,rho) (fun(rho).*(cos(th)*(R(1,1)*R(1,3)+R(2,1)*R(2,3))+sin(th)*(R(1,2)*R(1,3)+R(2,2)*R(2,3)))+rho.*(R(1,3).^2+R(2,3).^2-1)+R(3,3)*(etaX(th,rho)-RefP(3)));

        % Numerically find a set of rhoZeroVec that satisfy Fun_inter
        nTheta     = 13;
        thVec      = linspace(-pi,pi,nTheta);
        rhoZeroVec = zeros(1,nTheta);                
        for ii = 1:nTheta
            rhoZero        = fzero(@(rho)Fun_inter(thVec(ii),rho), 0);
            % I'm assuming the intersection stays within the section I am analyziing, therefore no need for the following line
            % rhoZero = max(min(rhoZero,NLFKgeom.(name{1}).rho2),NLFKgeom.(name{1}).rho1); % If the intersection goes below or above the limits, then saturate   
            rhoZeroVec(ii) = rhoZero;
        end
        rho2 = @(th)spline(thVec, rhoZeroVec, th); % Define a functional that interpolate rhoZeroVec    
        
    case 'linear'    
        q = NLFKgeom.(name{1}).q;
        m = NLFKgeom.(name{1}).m;
        
        % Selecet a set of x-points in the world frame to sample eta in the world frame and apply a linear regression
        nPoints = 13;
        xFit    = linspace([1 0 0 0]*b2w([0 0 0 1]')-q, [1 0 0 0]*b2w([0 0 0 1]')+q, nPoints);
        etaX    = par.etaFun(xFit, t);
        p       = polyfit(xFit, etaX, 1); % Linear regression
        p0      = p(1);
        p1      = p(2);

        % Find analytical intersection (it's the exact solution of Fun_inter if eta were linear)
        hFun = @(th) cos(th)*(R(1,1)*(R(1,3)+p0*R(3,3))+R(2,1)*R(2,3))   +   sin(th)*(R(1,2)*(R(1,3)+p0*R(3,3))+R(2,2)*R(2,3))   ;
        rho2 = @(th) (R(3,3)*(RefP(3)-p1-p0*RefP(1))-q*(hFun(th))) ./  (R(1,3).^2+R(2,3).^2-1+p0*R(1,3)*R(3,3)  +m*(hFun(th))   );

    case 'flat'  
        q = NLFKgeom.(name{1}).q;
        m = NLFKgeom.(name{1}).m;

        % Find analytical intersection (it's the exact solution of Fun_inter if eta were flat)
        A1   = R(1,1)*R(1,3)+R(2,1)*R(2,3);
        A2   = R(1,2)*R(1,3)+R(2,2)*R(2,3);
        rho2 = @(th) (R(3,3)*(RefP(3)       - flatLimit) -q*(A1*cos(th)+A2*sin(th))   )  ./   (R(1,3).^2+R(2,3).^2-1 +m*(A1*cos(th)+A2*sin(th))   );
end

end