function NLFKgeom = FUN_computeBodyProperties(NLFKgeom)
% This function computes total and wetter lateral surface, total and submberged volumes, and compares them to
% algebraic results. Moreover, The vertical position of the centre of buoyancy is also computed
%
% Please refer to the manual for the full set of equations that are solved herein
%
%   INPUT: 
%->       NLFKgeom = structure created by FUN_defineNLFKgeom
%   OUTPUT: 
%->       NLFKgeom = same as the input, with the addition of the vertical position of the centre of buoyancy zB
%
%
% Function written by Dr. Giuseppe Giorgi (ORCID: 0000-0002-1649-9438) under the H2020-MCSA-IF-2018 fellowship
% Grant agreement: 832140
% NLFK4ALL - OpenWave: Validation and Optimization of an Open-Source Novel Nonlinear Froude-Krylov Model for Advanced Design of Wave Energy Converters
% Last update: 09/11/2019

%% Lateral surface
    % SLat  = integral2(@(th,rho)fun(rho).*sqrt(1+dfun(rho).^2),-pi,pi,rho1,rho2);
    
    SLat = 0;
    SWet = 0;
    for iPatch = NLFKgeom.Patches.cylCoor
        name  = {['Patch_',num2str(iPatch)]};
        fun   = NLFKgeom.(name{1}).fun;
        dfun  = NLFKgeom.(name{1}).dfun;
        rho1  = NLFKgeom.(name{1}).rho1;
        rho2  = NLFKgeom.(name{1}).rho2;

        PatchSLat = integral2(@(th,rho)fun(rho).*sqrt(1+dfun(rho).^2),-pi,pi,rho1,rho2);                            % Lateral surface of the patch 
        % If only the wetted surface is considered, I stop at z=0 (World-frame)
        if ismember(iPatch, NLFKgeom.Patches.freeSurf) 
            rho2_SWL  = 0 - NLFKgeom.zRefP;
            PatchSWet = integral2(@(th,rho)fun(rho).*sqrt(1+dfun(rho).^2),-pi,pi,NLFKgeom.(name{1}).rho1,rho2_SWL); % Wetted surface of the patch 
        else
            PatchSWet = PatchSLat;
        end
        SLat = SLat + PatchSLat;
        SWet = SWet + PatchSWet;
    end

    
    
    if abs((NLFKgeom.SLat-SLat)/NLFKgeom.SLat)<=1e-5
        disp(['CORRECT! Lateral surface integral: ', num2str(SLat)])
    else
        fprintf('PROBLEMS for the lateral surface integral:\n\tAlgebraic solution: %g, numerical solution: %g\n',SLat,NLFKgeom.SLat)
    end

    if abs((NLFKgeom.SWet-SWet)/NLFKgeom.SWet)<=1e-5
        disp(['CORRECT! Wetted lateral surface integral: ', num2str(SWet)])
    else
        fprintf('PROBLEMS for the wetted lateral surface integral:\n\tAlgebraic solution: %g, numerical solution: %g\n',SWet,NLFKgeom.SWet)
    end

    

    %% Volumes
    % The volume can be computed in three equivalent alternative ways, along the x-, the y-, or the z-axis
    %
    % Vx  = -integral2(@(th,rho) -cos(th).*fun(rho)  .* fun(rho).*cos(th),-pi,pi,rho1,rho2)+0+0;
    % Vy  = -integral2(@(th,rho) -sin(th).*fun(rho)  .* fun(rho).*sin(th),-pi,pi,rho1,rho2)+0+0;
    % VzL = -integral2(@(th,rho) dfun(rho).*fun(rho) .* rho              ,-pi,pi,rho1,rho2);

    VTotx = 0;
    VToty = 0;
    VTotz = 0;
    
    VSubx = 0;
    VSuby = 0;
    VSubz = 0;

    for iPatch = NLFKgeom.Patches.cylCoor
        name    = {['Patch_',num2str(iPatch)]};
        fun     = NLFKgeom.(name{1}).fun;
        dfun    = NLFKgeom.(name{1}).dfun;
        rho1    = NLFKgeom.(name{1}).rho1;
        rho2    = NLFKgeom.(name{1}).rho2;
        normDir = NLFKgeom.(name{1}).normDir;

        PatchVTotx = - normDir*integral2(@(th,rho) -cos(th).*fun(rho)  .* fun(rho).*cos(th),-pi,pi,rho1,rho2); % Volume of the patch, computed along x
        PatchVToty = - normDir*integral2(@(th,rho) -sin(th).*fun(rho)  .* fun(rho).*sin(th),-pi,pi,rho1,rho2); % Volume of the patch, computed along y
        
% The integration in the z-direction of the profile of revolution (PatchVTotz) considers the volume of the projected profile onto x=0.
% Therefore, I have to add the volume of the cylinder at the bottom (PatchVTotz_Bot), 
% and subtract the volume of the cylinder at the top (PatchVTotz_Top) of the patch,

        PatchVTotz     = - normDir*integral2(@(th,rho) dfun(rho).*fun(rho) .* rho          ,-pi,pi,rho1,rho2);
        PatchVTotz_Bot = - normDir*( pi*fun(rho1)^2*rho1);
        PatchVTotz_Top = - normDir*(-pi*fun(rho2)^2*rho2);
        PatchVTotz     = PatchVTotz  + PatchVTotz_Bot + PatchVTotz_Top;                                        % Volume of the patch, computed along z
        
        % If only the submerged volume is considered, I stop at z=0 (World-frame)
        if ismember(iPatch, NLFKgeom.Patches.freeSurf)
            rho2_SWL   = 0 - NLFKgeom.zRefP;
            PatchVSubx = - normDir*integral2(@(th,rho) -cos(th).*fun(rho)  .* fun(rho).*cos(th),-pi,pi,rho1,rho2_SWL); % Submerged volume of the patch, computed along x
            PatchVSuby = - normDir*integral2(@(th,rho) -sin(th).*fun(rho)  .* fun(rho).*sin(th),-pi,pi,rho1,rho2_SWL); % Submerged volume of the patch, computed along y

% The integration in the z-direction of the profile of revolution (PatchVTotz) considers the volume of the projected profile onto x=0.
% Therefore, I have to add the volume of the cylinder at the bottom (PatchVTotz_Bot), 
% and subtract the volume of the cylinder at the top (PatchVTotz_Top) of the patch,
            PatchVSubz     = - normDir*integral2(@(th,rho) dfun(rho).*fun(rho) .* rho          ,-pi,pi,rho1,rho2_SWL);
            PatchVSubz_Bot = - normDir*( pi*fun(rho1)^2*rho1);
            PatchVSubz_Top = - normDir*(-pi*fun(rho2_SWL)^2*rho2_SWL);
            PatchVSubz     = PatchVSubz  + PatchVSubz_Bot + PatchVSubz_Top;                                        % Submerged volume of the patch, computed along z
        else
            PatchVSubx = PatchVTotx;
            PatchVSuby = PatchVToty;
            PatchVSubz = PatchVTotz;
        end
        
        VTotx = VTotx + PatchVTotx;
        VToty = VToty + PatchVToty;
        VTotz = VTotz + PatchVTotz;

        VSubx = VSubx + PatchVSubx;
        VSuby = VSuby + PatchVSuby;
        VSubz = VSubz + PatchVSubz;
        
    end

    
    
    
    
    if abs((NLFKgeom.VTot-VTotz)/NLFKgeom.VTot)<=1e-5 && abs(VTotx-VToty)<=1e-5*NLFKgeom.VTot && abs(VToty-VTotz)<=1e-5*NLFKgeom.VTot
        disp(['CORRECT! Volume integral: ', num2str(VTotz)])
    else
        fprintf('PROBLEMS for the volume integral:\n\tAlgebraic solution: %g, numerical solution: %g\n',VTot,NLFKgeom.VTot)
    end

    
    if abs((NLFKgeom.VSub-VSubz)/NLFKgeom.VSub)<=1e-5 && abs(VSubx-VSuby)<=1e-5*NLFKgeom.VSub && abs(VSuby-VSubz)<=1e-5*NLFKgeom.VSub
        disp(['CORRECT! Submerged volume integral: ', num2str(VSubz)])
    else
        fprintf('PROBLEMS for the sumberged volume integral:\n\tAlgebraic solution: %g, numerical solution: %g\n',VSubz,NLFKgeom.VSub)
    end


    
    
%% Centre of buoyancy
% zB=1/(2*Vsub)*(-integral2(@(th,rho) dfun(rho).*fun(rho).*rho.^2,-pi,pi,rho1,rho0) -pi*R1^2*rho1^2+pi*R0^2*rho0^2);

zB = 0;
for iPatch = NLFKgeom.Patches.cylCoor
    name  = {['Patch_',num2str(iPatch)]};
    fun   = NLFKgeom.(name{1}).fun;
    dfun  = NLFKgeom.(name{1}).dfun;
    rho1  = NLFKgeom.(name{1}).rho1;
    normDir = NLFKgeom.(name{1}).normDir;
    
    if ismember(iPatch, NLFKgeom.Patches.freeSurf)
        rho2 = 0 - NLFKgeom.zRefP;
    else
        rho2  = NLFKgeom.(name{1}).rho2;
    end
    
    zB = zB -normDir*integral2(@(th,rho) dfun(rho).*fun(rho).*rho.^2,-pi,pi,rho1,rho2);
    
    Bot = -normDir*( pi*fun(rho1)^2*rho1^2);
    Top = -normDir*(-pi*fun(rho2)^2*rho2^2);

    zB = zB + Bot + Top;
     
end
zB = 1/(2*VSubz)*zB;

zB = zB + NLFKgeom.zRefP; % Translate to the world coordinates

NLFKgeom.zB = zB;    

disp(['Center of buoyancy: ', num2str(zB)])
    
    


end

