function NLFKgeom = FUN_defineNLFKgeom(zCoG, zRefP, xz)
% FUN_defineNLFKgeom creates a .mat (NLFKgeom) file for the computation of 
% Nonlinear Froude-Krylov forces for axyisymmetric floating buoys, based on the
% method developed by Giuseppe Giorgi (ORCID: 0000-0002-1649-9438).
% The output geometry is described with respect to the body-fixed frame
% with the origin in the centre of gravity.
%
% For the notation adopted in this script, please refer to the attached
% manual.
% 
% In this version, arbitrary combinations of disks, cylinders and cones are
% allowed. Other shapes, although compatible to the framework, are not 
% considered for readibility of these demonstration scripts. 
%
% Each section is defined as "Patch":
%       2 horizontally aligned points define a disk;
%       2 vertically aligned points define the external surface of a cylinder;
%       2 not-aligned points define the external surface of a truncated cone;
% 
%   INPUT: 
%         Use a right-handed frame of reference with the origin at the
%         still water level, with z positive upwards and conciding with the
%         axis of revolution of the buoy (World-frame: WF)
%
%->       zCoG = vertical coordinate of the centre of gravity, in the WF
%->       zRefP = vertical coordinate of the reference point (origin of the BF), in the WF
%       
%->       xz   = (Nx2) vector, containing the sequence of points defining,
%         patch by patch, the entire profile of revolution, in the WF. It follows that
%         the number of patches is (N-1). Consider all positive x and z=0 
%         being the axis of revolution.
%
%         As a convention, z values are given in decreasing order for 
%         external surfaces (water on the outside), while in increasing 
%         order for internal surfaces (water on the inside).
%
%   EXAMPLE 1: xz = [2 1; 2 -5; 1 -5] defines a cylinder of radius 2, draft 5, 
%   and freeboard 1, composed of two pathes (one for the lateral surface and one for
%   the bottom disk)
%
%   EXAMPLE 2: xz = [2.5 1; 1.5 -5; 0 -5] defines a truncated cone of 
%   larger radius 2.5, smaller radius 1., draft 5, and freeboard 1. It is composed of 
%   two pathes (one for the lateral surface, and one for the bottom disk). 
%
%   EXAMPLE 3: xz = [2 1; 2 -5; 1 -5; 1 1] defines a hollow cylinder of 
%   external radius 2, internal radius 1, draft 5, and freeboard 1. It is composed of 
%   three pathes (one for the external lateral surface, one for the bottom 
%   disk, and one for the internal lateral surface-moonpool). 
%
%
%
% Function written by Dr. Giuseppe Giorgi (ORCID: 0000-0002-1649-9438) under the H2020-MCSA-IF-2018 fellowship
% Grant agreement: 832140
% NLFK4ALL - OpenWave: Validation and Optimization of an Open-Source Novel Nonlinear Froude-Krylov Model for Advanced Design of Wave Energy Converters
% Last update: 09/11/2019


%%

NLFKgeom.zCoG  = zCoG;
NLFKgeom.zRefP = zRefP;
NLFKgeom.xz    = xz;

%%
N                  =  size(xz,1)-1;           % Number of patches
NLFKgeom.Patches.N = N;

NLFKgeom.Patches.polarCoor = zeros(N,1);      % Patches with polar coordinates
NLFKgeom.Patches.cylCoor   = zeros(N,1);      % Patches with cylindrical coordinates

NLFKgeom.Patches.freeSurf  = zeros(N,1);      % Patches at the free surface

for ii = 1:N
    if xz(ii,2) == xz(ii+1,2)
        NLFKgeom.Patches.polarCoor(ii) = ii;
    else
        NLFKgeom.Patches.cylCoor(ii) = ii;
    end
    
    if any(xz([ii ii+1],2)>=0)
        NLFKgeom.Patches.freeSurf(ii) = ii;
    end
end

NLFKgeom.Patches.polarCoor = NLFKgeom.Patches.polarCoor(NLFKgeom.Patches.polarCoor ~= 0)';
NLFKgeom.Patches.cylCoor   = NLFKgeom.Patches.cylCoor(NLFKgeom.Patches.cylCoor ~= 0)';
NLFKgeom.Patches.freeSurf  = NLFKgeom.Patches.freeSurf(NLFKgeom.Patches.freeSurf ~= 0)';


%%
SLat = 0; % Lateral surface of the cylindrical/conical sections
SWet = 0; % Wetted lateral surface of the cylindrical/conical sections
VTot = 0; % Total volume
VSub = 0; % Submerged volume

for ii = 1:N
    
    name = {['Patch_',num2str(ii)]};
    if ismember(ii,NLFKgeom.Patches.cylCoor)
        % It's cylindrical coordinates

        x     = xz([ii ii+1],1);
        z     = xz([ii ii+1],2);
        [z,I] = sort(z,'ascend');
        x     = x(I);
        % If I(1)-I(2) > 0, it is an external surface and norm = +1; 
        % Otherwise, it is an internal surface and norm = -1.
        normDir                    = I(1) - I(2);
        NLFKgeom.(name{1}).normDir = normDir;
                
        m = (x(2) - x(1)) / (z(2) - z(1));          % Slope of the profile function
        q = x(1) - m*(z(1) - NLFKgeom.zRefP);       % Intercept of the profile function
        
        NLFKgeom.(name{1}).m    = m;
        NLFKgeom.(name{1}).q    = q;
        NLFKgeom.(name{1}).rho1 = z(1) - NLFKgeom.zRefP;     % Bottom limit of integration, in the body-fixed frame (BF)
        NLFKgeom.(name{1}).rho2 = z(2) - NLFKgeom.zRefP;     % Top limit of integration, in the body-fixed frame (BF)
        
        NLFKgeom.(name{1}).fun  = @(rho) m*rho + repmat(q, size(rho));                    % Profile function
        NLFKgeom.(name{1}).dfun = @(rho) repmat(m, size(rho));                            % Derivative of the profile function

        NLFKgeom.(name{1}).n1  = @(th,rho) -normDir*(m*rho + q).*cos(th);                         % Integrand for the surge direction
        NLFKgeom.(name{1}).n2  = @(th,rho) -normDir*(m*rho + q).*sin(th);                         % Integrand for the sway direction
        NLFKgeom.(name{1}).n3  = @(th,rho)  normDir*(m*rho + q).*m;                               % Integrand for the heave direction
        NLFKgeom.(name{1}).n4  = @(th,rho)  normDir*(m*rho + q).*sin(th).*((m*rho + q).*m + rho); % Integrand for the roll direction
        NLFKgeom.(name{1}).n5  = @(th,rho) -normDir*(m*rho + q).*cos(th).*((m*rho + q).*m + rho); % Integrand for the pitch direction

        % Compute geometrical properties, useful for future checks
        PatchSLat = pi*(x(2) + x(1))*sqrt((x(2) - x(1))^2 + (z(2) - z(1))^2);     % Lateral surface of the revolution of the patch's profile
        PatchVTot = normDir * pi*(z(2) - z(1))/3 * (x(2)^2 + x(1)^2 + x(2)*x(1)); % Volume of the revolution of the patch's profile, negative if the hollow part is considered
        
        if z(2)<0
            PatchSWet = PatchSLat;
            PatchVWet = PatchVTot;
        else
            z2_0 = 0;                                                                 % Still water level, in the WF
            x2_0 = m*(z2_0 - NLFKgeom.zRefP) + q;                                     % Radius at the still water level
            NLFKgeom.(name{1}).RadiusAtSWL = x2_0;
            PatchSWet = pi*(x2_0 + x(1))*sqrt((x2_0 - x(1))^2 + (z2_0 - z(1))^2);     % Lateral surface of the revolution of the patch's profile
            PatchVWet = normDir * pi*(z2_0 - z(1))/3 * (x2_0^2 + x(1)^2 + x2_0*x(1)); % Volume of the revolution of the patch's profile, negative if the hollow part is considered
        end
        SLat = SLat + PatchSLat;
        SWet = SWet + PatchSWet;
        VTot = VTot + PatchVTot;
        VSub = VSub + PatchVWet;
    else
        % It's polar coordinates
        z0 = xz(ii,2) - NLFKgeom.zRefP;                  % Vertical position of the centre of the disk

        NLFKgeom.(name{1}).z0   = z0;
        NLFKgeom.(name{1}).r1   = min(xz([ii ii+1],1)); % Internal radius
        NLFKgeom.(name{1}).r2   = max(xz([ii ii+1],1)); % External radius

        NLFKgeom.(name{1}).fun = @(r) r;

        NLFKgeom.(name{1}).n1 = @(th,r) zeros(size(th)); % Integrand for the surge direction
        NLFKgeom.(name{1}).n2 = @(th,r) zeros(size(th)); % Integrand for the sway direction
        NLFKgeom.(name{1}).n3 = @(th,r) r;               % Integrand for the heave direction
        NLFKgeom.(name{1}).n4 = @(th,r) r.^2.*sin(th);   % Integrand for the roll direction
        NLFKgeom.(name{1}).n5 = @(th,r) -r.^2.*cos(th);  % Integrand for the pitch direction
    end



end

NLFKgeom.SLat = SLat;
NLFKgeom.SWet = SWet;
NLFKgeom.VTot = VTot;
NLFKgeom.VSub = VSub;



%% Compute the area and the outer and internal radii at the still water level (SWL)

for jj = 1:length(NLFKgeom.Patches.freeSurf)
    ii = NLFKgeom.Patches.freeSurf(jj);
    
    if jj==1
        name              = {['Patch_',num2str(ii)]};
        NLFKgeom.Rout_SWL = NLFKgeom.(name{1}).RadiusAtSWL;
        Area_SWL          = pi*NLFKgeom.Rout_SWL^2;             % Cross-sectional area of the outer patch
        NLFKgeom.Rin_SWL  = 0;
    else
        name               = {['Patch_',num2str(ii)]};
        NLFKgeom.Rin_SWL   = NLFKgeom.(name{1}).RadiusAtSWL;
        Area_SWL           = Area_SWL - pi*NLFKgeom.Rin_SWL^2;  % If there is a moonpool, substract the area to have only the annular area in total
    end
end
NLFKgeom.Area_SWL = Area_SWL;

end

