function FUN_plot3D(NLFKgeom, states, VIEW, etaFun, t)
% This function can plot the given axisymmetric geometry and a free surface elevation in the following frames:
%
% WF   = World-frame, right-handed, z positive upwards, with origin at the still water level
% BF   = Body-frame, right-handed, z positive upwards and conciding with the WF z-axis, with origin at the center of gravity
% CoG  = Center of Gravity, on the axis of the axisymmetric floater
% RefP = Reference point, origin of the BF, on the axis of the axisymmetric floater
% states = [xG, yG, zG, PHI, THETA, PSI] = 6-degree of freedom displacements of the RefP in the WF
% Angles according to the3-2-1 Euler rotation convention
% 
%   INPUT:
%->       NLFKgeom = file containing all geom characteristics
%->       states   = [xR, yR, zR, PHI, THETA, PSI] = 6-degree of freedom displacements of the RefP in the WF
%->       VIEW     = Aximuth and elevation angles of the 3D view
%->       etaFun   = Function for the free surface elevation: @(x, t) etaAmp*cos(etaFreq*t - etaNumber*x)
%->       t        = time instant for the etaFun evaluation (if t=nan, no surface elevation is plottet)
%
%
%
%
%
% Function written by Dr. Giuseppe Giorgi (ORCID: 0000-0002-1649-9438) under the H2020-MCSA-IF-2018 fellowship
% Grant agreement: 832140
% NLFK4ALL - OpenWave: Validation and Optimization of an Open-Source Novel Nonlinear Froude-Krylov Model for Advanced Design of Wave Energy Converters
% Last update: 09/11/2019
%%
warning('off') % The 3D-plotting delaunayTriangulation function may cause warning for duplicate data points, which are not a problem

xR    = states(1);               % x displacement of the RefP, in the WF
yR    = states(2);               % y displacement of the RefP, in the WF
zR    = states(3);               % z displacement of the RefP, in the WF
PHI   = states(4);               % rotation about an axis parallel to the x-axis, passing through the RefP
THETA = states(5);               % rotation about an axis parallel to the y-axis, passing through the RefP
PSI   = states(6);               % rotation about an axis parallel to the z-axis, passing through the RefP

%%
% Notation:
% The position of one generic point P can be expressed in the WF or the BF
% P_WF is P expressed in the WF
% P_BF is P expressed in the BF

WForigin_WF          = [0,   0,                  0]';  % Origin of WF, in the WF coordinates... it's fixed at [0,0,0] by definition
RefP_restPosition_WF = [0,   0,      NLFKgeom.zRefP]';  % Position of the CoG, before displacement, in the WF coordinates
RefP_WF              = [xR, yR, zR + NLFKgeom.zRefP]';  % Position of the CoG, after displacement, in the WF coordinates

Rx = [1 0         0        ; 
      0 cos(PHI) -sin(PHI) ; 
      0 sin(PHI)  cos(PHI)];         % rotation matrix around x
  
Ry = [cos(THETA) 0 sin(THETA) ; 
      0          1 0          ; 
     -sin(THETA) 0 cos(THETA)];     % rotation matrix around y
 
Rz = [cos(PSI) -sin(PSI) 0 ;
      sin(PSI)  cos(PSI) 0 ; 
      0         0        1];         % rotation matrix around z

% The order of rotation is important! There are different conventions. This is often used for marine applications
% [Fossen, Handbook of marine craft hydrodynamics and motion control, 2011]

RotPure = Rz*Ry*Rx;                   % Pure rotation    matrix in 3D: XYZ  = [x;y;z]  , such that WF = Rot*BF
R       = [RotPure RefP_WF; 0 0 0 1];  % Roto-translation matrix in 4D: XYZ1 = [x;y;z;1], such that WF = R*BF
RInv    = inv(R);

% The 4D notation is handy for roto-translations
% Any 4D-position vector is composed by the 3D-position vector in the first trhee rows, and a 1 in the fourth row

%%
b2w = @(XYZ1) R*XYZ1;    % transformation from XYZ1_BF to XYZ1_WF
w2b = @(XYZ1) RInv*XYZ1; %#ok<MINV> % transformation from XYZ1_WF to XYZ1_BF

WForigin_BF          = w2b([WForigin_WF; 1]);          % Origin of WF, in the WF coordinates
RefP_restPosition_BF = w2b([RefP_restPosition_WF; 1]); % Position of the RefP, before displacement, in the BF coordinates
RefP_BF              = w2b([RefP_WF; 1]);              % Position of the RefP, after displacement, in the BF coordinates (computed, but it's [0 0 0]' by definition)

%%
Ltot = max(NLFKgeom.xz(:,2)) - min(NLFKgeom.xz(:,2));                                                           % Total length of the floater
Axis_BF = [0                                               , 0; 
           0                                               , 0; 
           min(NLFKgeom.xz(:,2)) - NLFKgeom.zRefP - 0.2*Ltot, max(NLFKgeom.xz(:,2)) - NLFKgeom.zRefP+0.2*Ltot; 
           1                                               , 1];                                                % Axis of the floater, in the BF

Axis_WF = b2w(Axis_BF);

%%

figure;

% Plotting lateral surfaces (cylindrical coordinates)
for iPatch = NLFKgeom.Patches.cylCoor
    name = {['Patch_',num2str(iPatch)]};
    fun  = NLFKgeom.(name{1}).fun;

    % Axisymmetric geometry in the BF
    BF = @(th,rho) [fun(rho).*cos(th);
                    fun(rho).*sin(th);
                    rho; 
                    ones(size(th))];  % transformation from Cylindrical_BF to XYZ1_BF 

    % Axisymmetric geometry in the WF
    WF = @(th,rho) b2w(BF(th,rho));   % transformation from Cylindrical_BF to XYZ1_WF 

    thVec    = linspace(-pi, pi, 21); 
    rhoVec   = linspace(NLFKgeom.(name{1}).rho1,NLFKgeom.(name{1}).rho2,7);
    [RHO,TH] = meshgrid(rhoVec,thVec);

    XYZbf = BF(TH(:)',RHO(:)'); % Vector of points in the BF (4xn)
    XYZwf = WF(TH(:)',RHO(:)'); % Vector of points in the WF (4xn)

%%
    subplot(1,2,1); hold on
    plot3(XYZwf(1,:),XYZwf(2,:),XYZwf(3,:),'c','LineWidth',1.5)

    DTwf    = delaunayTriangulation(XYZwf(1:3,:)');
    [Cbf,~] = convexHull(DTwf);

    trisurf(Cbf,DTwf.Points(:,1),DTwf.Points(:,2),DTwf.Points(:,3),'LineStyle','none','FaceColor','blue','FaceAlpha',0.1)
    axis equal 


    subplot(1,2,2); hold on
    plot3(XYZbf(1,:),XYZbf(2,:),XYZbf(3,:),'g','LineWidth',1.5)

    DTbf    =delaunayTriangulation(XYZbf(1:3,:)');
    [Cbf,~] = convexHull(DTbf);

    trisurf(Cbf,DTbf.Points(:,1),DTbf.Points(:,2),DTbf.Points(:,3),'LineStyle','none','FaceColor','yellow','FaceAlpha',0.1)
    axis equal 

end



% Plotting disks (polar coordinates)
for iPatch = NLFKgeom.Patches.polarCoor
    name = {['Patch_',num2str(iPatch)]};
    z0   = NLFKgeom.(name{1}).z0;

    % Axisymmetric geometry in the BF
    BF = @(th,r) [r.*cos(th);
                  r.*sin(th);
                  z0*ones(size(th)); 
                  ones(size(th))];    % transformation from Cylindrical_BF to XYZ1_BF 

    % Axisymmetric geometry in the WF
    WF = @(th,r) b2w(BF(th,r));       % transformation from Cylindrical_BF to XYZ1_WF 

    thVec    = linspace(-pi, pi, 21); 
    rVec     = linspace(NLFKgeom.(name{1}).r1,NLFKgeom.(name{1}).r2,4);
    [RAD,TH] = meshgrid(rVec,thVec);

    XYZbf = BF(TH(:)',RAD(:)'); % Vector of points in the BF (4xn)
    XYZwf = WF(TH(:)',RAD(:)'); % Vector of points in the WF (4xn)

%%
    subplot(1,2,1); hold on
    plot3(XYZwf(1,:),XYZwf(2,:),XYZwf(3,:),'c','LineWidth',1.5)
    
    X = reshape(XYZwf(1,:),size(TH));
    Y = reshape(XYZwf(2,:),size(TH));
    Z = reshape(XYZwf(3,:),size(TH));
    surf(X,Y,Z,'LineStyle','none','FaceColor','b','FaceAlpha',0.1)
    grid on
    axis equal 

    
    subplot(1,2,2); hold on
    plot3(XYZbf(1,:),XYZbf(2,:),XYZbf(3,:),'g','LineWidth',1.5)
    
    X = reshape(XYZbf(1,:),size(TH));
    Y = reshape(XYZbf(2,:),size(TH));
    Z = reshape(XYZbf(3,:),size(TH));
    surf(X,Y,Z,'LineStyle','none','FaceColor','y','FaceAlpha',0.1)
    axis equal 
    
end

% World-frame coordinate
subplot(1,2,1)
plot3(WForigin_WF(1),         WForigin_WF(2),         WForigin_WF(3),'k+')          % Origin of WF, expressed in WF. By definition is (0,0,0)
plot3(RefP_restPosition_WF(1), RefP_restPosition_WF(2), RefP_restPosition_WF(3),'kx')  % CoG rest position, expressed in WF
plot3(RefP_WF(1),              RefP_WF(2),              RefP_WF(3),'ro')               % CoG displaced position, expressed in WF

% Displacement vector of the CoG, expressed in WF
plot3([RefP_restPosition_WF(1) RefP_WF(1)], [RefP_restPosition_WF(2) RefP_WF(2)], [RefP_restPosition_WF(3) RefP_WF(3)],'r-')

line(Axis_WF(1,:),            Axis_WF(2,:),           Axis_WF(3,:),'LineStyle','-.') % Axis of the floater, expressed in WF
view(VIEW)
grid on
axis equal 


% Body-frame coordinate
subplot(1,2,2)
plot3(WForigin_BF(1),         WForigin_BF(2),         WForigin_BF(3),'k+')          % Origin of WF, expressed in BF
plot3(RefP_restPosition_BF(1), RefP_restPosition_BF(2), RefP_restPosition_BF(3),'kx')  % CoG rest position, expressed in BF (also origin of BF by definition)
plot3(RefP_BF(1),              RefP_BF(2),              RefP_BF(3),'ro')               % CoG displaced position, expressed in BF

% Displacement vector of the CoG, expressed in BF
plot3([RefP_restPosition_BF(1) RefP_BF(1)], [RefP_restPosition_BF(2) RefP_BF(2)], [RefP_restPosition_BF(3) RefP_BF(3)],'r-')

line(Axis_BF(1,:),Axis_BF(2,:),Axis_BF(3,:),'LineStyle','-.') % Axis of the floater, expressed in BF
view(VIEW)
grid on
axis equal 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isnan(t)
    % Plot free surface elevation
    subplot(1,2,1)
    XLIM = get(gca,'XLim');
    YLIM = get(gca,'YLim');
    
    Xe = linspace(XLIM(1)-0.5*diff(XLIM), XLIM(2)+0.5*diff(XLIM), 20);
    Ye = linspace(YLIM(1)-0.5*diff(YLIM), YLIM(2)+0.5*diff(YLIM), 5);

    [XXe,YYe] = meshgrid(Xe,Ye);

    Xe   = XXe(:)';
    Ye   = YYe(:)';
    Ee   = etaFun(Xe, t);
    EEwf = reshape(Ee,size(XXe));

    XYEwf = [Xe;
             Ye;
             Ee;
             ones(size(Ee))];             % Free surface, expressed in the WF

    XYEbf = w2b(XYEwf);
    xxe   = reshape(XYEbf(1,:),size(XXe));
    yye   = reshape(XYEbf(2,:),size(XXe));
    EEbf  = reshape(XYEbf(3,:),size(XXe)); % Free surface, expressed in the BF


    subplot(1,2,1)
    surf(XXe,YYe,EEwf,'EdgeColor',[0 0.4470 0.7410],'FaceColor','blue','FaceAlpha',0.1)

    subplot(1,2,2)
    surf(xxe,yye,EEbf,'EdgeColor',[0 0.4470 0.7410],'FaceColor','blue','FaceAlpha',0.1)
    

    %%
    for iPatch = NLFKgeom.Patches.freeSurf

        name = {['Patch_',num2str(iPatch)]};
        fun  = NLFKgeom.(name{1}).fun;
        BF   = @(th,rho) [fun(rho).*cos(th);  fun(rho).*sin(th); rho; ones(size(th))];

        par.etaFun = etaFun;

        intMod     = {'exact','linear','flat'};
        lineStyle  = {'r-','b--','k:'};

        for ii = 1:length(intMod)
            flatLimit = etaFun(xR, t);
            rho2 = FUN_findIntersection(intMod{ii}, par, NLFKgeom, states, t, R, iPatch, flatLimit);

            XYZbf_int = BF(thVec, rho2(thVec));
            XYZwf_int = b2w(XYZbf_int); 
            
            subplot(1,2,1)
            plot3(XYZwf_int(1,:),XYZwf_int(2,:),XYZwf_int(3,:),[lineStyle{ii}])
            
            subplot(1,2,2)
            plot3(XYZbf_int(1,:),XYZbf_int(2,:),XYZbf_int(3,:),[lineStyle{ii}])
        end
    end    
    
end

% If all states are zero (rest position), I also plot the profile of revolution, given by the user as an input
if sum(abs(states))==0
    subplot(1,2,1); hold on
    plot3(NLFKgeom.xz(:,1),0*NLFKgeom.xz(:,1),NLFKgeom.xz(:,2),'k')
end

%%
subplot(1,2,1)
title('World-frame')
xlabel('$x$','Interpreter','latex');
ylabel('$y$','Interpreter','latex');
zlabel('$z$','Interpreter','latex');

subplot(1,2,2)
title('Body-fixed frame');
xlabel('$\hat{x}$','Interpreter','latex');
ylabel('$\hat{y}$','Interpreter','latex');
zlabel('$\hat{z}$','Interpreter','latex');


spreadfigures

warning('on')
end























%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function rho2 = FUN_findIntersection(modIntersect, par, NLFKgeom, states, t, R, iPatch, flatLimit)
% FUN_findIntersection finds finds the interstection, in the body-fixed frame between the free surface elevation and one patch
% For simplicity, it is assumed that the free surface elevation is fully-contained in one patch
%
% Please refer to the manual for the explicit functions for computing the intersection
%
% INPUT:
%->     modIntersect: can be 'exact', 'linear', 'flat'
%->     par         : structure containing the function for the surface elevation -> par.etaFun
%->     NLFKgeom    : structuer with geometrical charachteristics of the floater
%->     states      : displacements of the floater, in the world-frame
%->     t           : time instanct
%->     R           : 4D rotation matrix
%->     iPatch      : patch index
%->     flatLimit   : horizontal intersection, in the WF, used if modIntersect = 'flat'
%
% OUTPUT:
%->     rho2: function of theta that returns the parametric intersection in BF
%

RefP  = [states(1), states(2), states(3) + NLFKgeom.zRefP]'; % Position of the RefP, in the WF
b2w  = @(XYZ1) R*XYZ1;                                     % Function Body-frame(XYZ1) to World-frame(XYZ1)
name = {['Patch_',num2str(iPatch)]};
fun  = NLFKgeom.(name{1}).fun;   
switch modIntersect
    case 'exact'
        BF   = @(th,rho) [fun(rho).*cos(th);  fun(rho).*sin(th); rho; ones(size(th))];     % function from (th,rho) to XYZ1_bf 
        X    = @(th,rho) [1 0 0 0]*b2w(BF(th,rho));                                        % function from (th,rho) to just x in the world frame
        etaX = @(th,rho) par.etaFun(X(th,rho), t);                                         % function from (th,rho) to eta in the world frame

        % The zeros of Fun_inter define the exact intersection between body and wave
        Fun_inter = @(th,rho) (fun(rho).*(cos(th)*(R(1,1)*R(1,3)+R(2,1)*R(2,3))+sin(th)*(R(1,2)*R(1,3)+R(2,2)*R(2,3)))+rho.*(R(1,3).^2+R(2,3).^2-1)+R(3,3)*(etaX(th,rho)-RefP(3)));

        % Numerically find a set of rhoZeroVec that satisfy Fun_inter
        nTheta     = 13;
        thVec      = linspace(-pi,pi,nTheta);
        rhoZeroVec = zeros(1,nTheta);                
        for ii = 1:nTheta
            rhoZero        = fzero(@(rho)Fun_inter(thVec(ii),rho), 0);
            % I'm assuming the intersection stays within the section I am analyziing, therefore no need for the following line
            % rhoZero = max(min(rhoZero,NLFKgeom.(name{1}).rho2),NLFKgeom.(name{1}).rho1); % If the intersection goes below or above the limits, then saturate   
            rhoZeroVec(ii) = rhoZero;
        end
        rho2 = @(th)spline(thVec, rhoZeroVec, th); % Define a functional that interpolate rhoZeroVec    
        
    case 'linear'    
        q = NLFKgeom.(name{1}).q;
        m = NLFKgeom.(name{1}).m;
        
        % Selecet a set of x-points in the world frame to sample eta in the world frame and apply a linear regression
        nPoints = 13;
        xFit    = linspace([1 0 0 0]*b2w([0 0 0 1]')-q, [1 0 0 0]*b2w([0 0 0 1]')+q, nPoints);
        etaX    = par.etaFun(xFit, t);
        p       = polyfit(xFit, etaX, 1); % Linear regression
        p0      = p(1);
        p1      = p(2);

        % Find analytical intersection (it's the exact solution of Fun_inter if eta were linear)
        hFun = @(th) cos(th)*(R(1,1)*(R(1,3)+p0*R(3,3))+R(2,1)*R(2,3))   +   sin(th)*(R(1,2)*(R(1,3)+p0*R(3,3))+R(2,2)*R(2,3))   ;
        rho2 = @(th) (R(3,3)*(RefP(3)-p1-p0*RefP(1))-q*(hFun(th))) ./  (R(1,3).^2+R(2,3).^2-1+p0*R(1,3)*R(3,3)  +m*(hFun(th))   );

    case 'flat'  
        q = NLFKgeom.(name{1}).q;
        m = NLFKgeom.(name{1}).m;

        % Find analytical intersection (it's the exact solution of Fun_inter if eta were flat)
        A1   = R(1,1)*R(1,3)+R(2,1)*R(2,3);
        A2   = R(1,2)*R(1,3)+R(2,2)*R(2,3);
        rho2 = @(th) (R(3,3)*(RefP(3)       - flatLimit) -q*(A1*cos(th)+A2*sin(th))   )  ./   (R(1,3).^2+R(2,3).^2-1 +m*(A1*cos(th)+A2*sin(th))   );
end
        
end





