% --> function [A,B,Fe,Ffk,Fd,Hydrostatic] = FUN_NemohAxi(FileName,inputGeom,w,dir,depth,rhoWater)
%
%
% Inputs : 
%   - FileName    : Directory name to store results
%   - inputGeom   : structure containing, for each body
    %   - name
    %   - r         : array of radial coordinates
    %   - z         : array of vertical coordinates, all negative and sorted descending (from zero downwards)
    %   - n         : number of points for discretisation
    %   - n_ang     : number of points for angular discretisaztion
    %   - DoF       : array of the desired degrees of freedom
    %   - RefP      : coordinates of the reference point [xR yR zR]
    %   - Npan      : target for number of panels
%   - w         : wave frequencies [rad/s]
%   - dir       : wave directions [deg]
%   - depth     : water depth (>0) [m]
%   - rhoWater  : water density
%
% Outputs : 
%   - A     : added mass
%   - B     : radiation damping
%   - Fe    : wave excitation force
%   - Ffk   : froyde-krylov component of the wave excitation force
%   - Fd    : diffraction component of the wave excitation force
%   - Hydrostatic: structure containing
    %   - Mass      : mass of body
    %   - Kh        : (6 x 6) hydrostatic stiffness matrix
    %   - CoB       : coordinates of buoyancy center
    %   - mesh      : structure containing mesh informations
% 
%
% Modified by Giuseppe Giorgi, Politecnico di Torino, Italy,
%          from the original matlab function axiMesh.m and nemoh.m developed by Aurlien Babarit, LHEEA Lab.
%%
function [A, B, Fe, Ffk, Fd, Hydrostatic] = FUN_NemohAxi(FileName, inputGeom, w, dir, depth, rhoWater)
g   = 9.806;            % Acceleration of gravity
nBi = length(inputGeom);

%%
% Starting to write Nemoh input file
system(['mkdir ',FileName]);
system(['mkdir ',FileName,filesep,'mesh']);
system(['mkdir ',FileName,filesep,'results']);

fid = fopen([FileName,filesep,'Nemoh.cal'],'w');
fprintf(fid,'--- Environment ------------------------------------------------------------------------------------------------------------------ \n');
fprintf(fid,'%f				! RHO 			! KG/M**3 	! Fluid specific volume \n',rhoWater);
fprintf(fid,'%f				! G			    ! M/S**2	! Gravity \n',g);
fprintf(fid,'%f             ! DEPTH			! M         ! Water depth\n',depth);
fprintf(fid,'0.	0.          ! XEFF YEFF		! M         ! Wave measurement point\n');
fprintf(fid,'--- Description of floating bodies -----------------------------------------------------------------------------------------------\n');
fprintf(fid,'%g				! Number of bodies\n',nBi);
status = fclose(fid);

%% Iteration for each body

Hydrostatic(1,nBi).name = []; 
for iBi = 1:nBi
    name    = inputGeom(iBi).name;
    r       = inputGeom(iBi).r;
    z       = inputGeom(iBi).z;
    n_ang   = inputGeom(iBi).n_ang;
    DoF     = inputGeom(iBi).DoF;
    RefP    = inputGeom(iBi).RefP;
    Npan    = inputGeom(iBi).Npan;
    nDoF    = length(DoF); 

    Hydrostatic(1,nBi).mesh.NpanTarget = Npan;
    Hydrostatic(1,nBi).mesh.profile_r  = r;
    Hydrostatic(1,nBi).mesh.profile_z  = z;
    Hydrostatic(1,nBi).mesh.n_ang      = n_ang;

    n = length(r);

%     status = close('all');
    ntheta = n_ang; 
    theta  = linspace(0,pi,ntheta);


    % Calcul des sommets du maillage
    x = zeros(1,ntheta*n);
    y = zeros(1,ntheta*n);
    z = repmat(z,1,ntheta);

    nx = 0;
    for jj=1:ntheta
        for ii=1:n    
            nx = nx+1;                          % number of points

            x(nx) = r(ii)*cos(theta(jj));
            y(nx) = r(ii)*sin(theta(jj));
        end
    end

    % Calcul des facettes
    nf = 0;
    NN = zeros(4,(n-1)*(ntheta-1));             % Matrix of vertex of rectangular panels of the initial mesh
    for ii=1:n-1
        for jj=1:ntheta-1
            nf = nf+1;                          % number of faces

            NN(1,nf) = ii+n*(jj-1);
            NN(2,nf) = ii+1+n*(jj-1);
            NN(3,nf) = ii+1+n*jj;
            NN(4,nf) = ii+n*jj;
        end
    end

    % Affichage de la description du maillage
    nftri = 0;
    tri   = zeros(2*nf,3);                              % Matrix of vertex of triangular panels of the initial mesh
    for ii=1:nf
        nftri        = nftri+1;
        tri(nftri,:) = [NN(1,ii) NN(2,ii) NN(3,ii)];

        nftri        = nftri+1;
        tri(nftri,:) = [NN(1,ii) NN(3,ii) NN(4,ii)];
    end

    figure;
    trimesh(tri,x,y,z,zeros(nx,1));
    axis equal
    title('Characteristics of the discretisation');
    box on
    grid on

    fprintf('\n --> Number of nodes             : %g',nx);
    fprintf('\n --> Number of panels (max 2000) : %g \n',nf);

    % Creation des fichiers de calcul du maillage
    fid = fopen('Mesh.cal','w');
    fprintf(fid,'%s \n',name);
    fprintf(fid,'1 \n 0. 0. \n ');
    fprintf(fid,'%f %f %f \n',RefP);
    fprintf(fid,'%g \n 2 \n 0. \n 1.\n',Npan);
    fprintf(fid,'%f \n %f \n',[rhoWater g]);
    status = fclose(fid);

    fid = fopen('ID.dat','w');
    fprintf(fid,['% g \n',FileName,' \n'],length(FileName));
    status = fclose(fid);

    fid = fopen([FileName,filesep,'mesh',filesep,name],'w');
    fprintf(fid,'%g \n',nx);
    fprintf(fid,'%g \n',nf);
    for ii=1:nx
        fprintf(fid,'%E %E %E \n',[x(ii) y(ii) z(ii)]);
    end
    for ii=1:nf
        fprintf(fid,'%g %g %g %g \n',NN(:,ii)');
    end
    status = fclose(fid);

    % Raffinement automatique du maillage et calculs hydrostatiques
    OS = isunix;
    if OS == 1
        system('mesh >Mesh.log');
    else
        system('.\Mesh\Mesh.exe >Mesh\Mesh.log');
    end

    % Visualisation du maillage
    clear x y z NN nx nf nftri tri uu vv ww;
    fid   = fopen([FileName,filesep,'mesh',filesep,name,'.tec'],'r');
    ligne = fscanf(fid,'%s',2);
    nx    = fscanf(fid,'%g',1);
    ligne = fscanf(fid,'%s',2);
    nf    = fscanf(fid,'%g',1);
    ligne = fgetl(fid);

    fprintf('\n Characteristics of the mesh for Nemoh \n');
    fprintf('\n --> Number of nodes : %g',nx);
    fprintf('\n --> Number of panels : %g\n \n',nf);

    % Reading points
    for ii = 1:nx
        ligne = fscanf(fid,'%f',6);
        x(ii) = ligne(1);
        y(ii) = ligne(2);
        z(ii) = ligne(3);
    end

    % Reading faces
    for ii = 1:nf
        ligne    = fscanf(fid,'%g',4);
        NN(1,ii) = ligne(1);
        NN(2,ii) = ligne(2);
        NN(3,ii) = ligne(3);
        NN(4,ii) = ligne(4);
    end
    nftri = 0;
    for ii = 1:nf
        nftri        = nftri+1;
        tri(nftri,:) = [NN(1,ii) NN(2,ii) NN(3,ii)];
        nftri        = nftri+1;
        tri(nftri,:) = [NN(1,ii) NN(3,ii) NN(4,ii)];
    end
    ligne = fgetl(fid);
    ligne = fgetl(fid);

    % Reading normal vectors
    xu = zeros(1,nf); yv = xu; zw = xu; uu = xu; vv = xu; ww = xu;
    for ii = 1:nf    
        ligne  = fscanf(fid,'%g %g',6);
        xu(ii) = ligne(1);
        yv(ii) = ligne(2);
        zw(ii) = ligne(3);
        uu(ii)  = ligne(4);
        vv(ii)  = ligne(5);
        ww(ii)  = ligne(6);
    end
    status = fclose(fid);

    figure;
    trimesh(tri,x,y,z);         % plot mesh
    hold on;
    quiver3(xu,yv,zw,uu,vv,ww); % plot normal vectos 
    axis equal
    title('Mesh for Nemoh');
    box on
    grid on


    Hydrostatic(1,nBi).mesh.x    = x;
    Hydrostatic(1,nBi).mesh.y    = y;
    Hydrostatic(1,nBi).mesh.z    = z;
    Hydrostatic(1,nBi).mesh.NN   = NN;
    Hydrostatic(1,nBi).mesh.tri  = tri;
    Hydrostatic(1,nBi).mesh.Npan = length(tri);

    Kh  = zeros(6,6);
    fid = fopen([FileName,filesep,'mesh',filesep,'KH.dat'],'r');
    for ii = 1:6   
        ligne    = fscanf(fid,'%g %g',6);
        Kh(ii,:) = ligne;
    end
    status = fclose(fid);
    movefile([FileName,filesep,'mesh',filesep,'KH.dat'],[FileName,filesep,'mesh',filesep,name,'KH.dat']);

    % clear XB YB ZB Mass WPA Inertia
    Inertia = zeros(6,6);
    fid     = fopen([FileName,filesep,'mesh',filesep,'Hydrostatics.dat'],'r');
    ligne   = fscanf(fid,'%s',2);
    XB      = fscanf(fid,'%f',1);
    ligne   = fgetl(fid);
    ligne   = fscanf(fid,'%s',2);
    YB      = fscanf(fid,'%f',1);
    ligne   = fgetl(fid);
    ligne   = fscanf(fid,'%s',2);
    ZB      = fscanf(fid,'%f',1);
    ligne   = fgetl(fid);
    ligne   = fscanf(fid,'%s',2);
    Mass    = fscanf(fid,'%f',1)*rhoWater;
    ligne   = fgetl(fid);
    ligne   = fscanf(fid,'%s',2);
    WPA     = fscanf(fid,'%f',1);
    status  = fclose(fid);
    movefile([FileName,filesep,'mesh',filesep,'Hydrostatics.dat'],[FileName,filesep,'mesh',filesep,name,'Hydrostatics.dat']);
    clear ligne

    fid = fopen([FileName,filesep,'mesh',filesep,'Inertia_hull.dat'],'r');
    for ii=1:3
        ligne             = fscanf(fid,'%g %g',3);
        Inertia(ii+3,4:6) = ligne;
    end
    Inertia(1,1) = Mass;
    Inertia(2,2) = Mass;
    Inertia(3,3) = Mass;
    status  = fclose(fid);
    movefile([FileName,filesep,'mesh',filesep,'Inertia_hull.dat'],[FileName,filesep,'mesh',filesep,name,'Inertia_hull.dat']);

    %%

    Hydrostatic(1,nBi).Mass        = Mass;
    Hydrostatic(1,nBi).WPA         = WPA;
    Hydrostatic(1,nBi).KH          = Kh;
    Hydrostatic(1,nBi).CoB         = [XB,YB,ZB];
    
    %%
    fid = fopen([FileName,filesep,'Nemoh.cal'],'a');
    fprintf(fid,'--- Body %d -----------------------------------------------------------------------------------------------------------------------\n',iBi);
    if isunix 
        fprintf(fid,['''',FileName,filesep,'mesh',filesep,name,'.dat''		! Name of mesh file\n']);
    else
        fprintf(fid,[FileName,'\\mesh\\',name,'.dat      ! Name of mesh file\n']);
    end
    fprintf(fid,'%g %g			! Number of points and number of panels 	\n',nx,nf);

    
    fprintf(fid,'%d                          ! Number of degrees of freedom\n',nDoF);
    for iDoF = 1:nDoF
        switch DoF(iDoF)
            case 1
                fprintf(fid,'1 1. 0.	0. 0. 0. 0.		! Surge\n');
            case 2
                fprintf(fid,'1 0. 1.    0. 0. 0. 0.		! Sway\n');
            case 3
                fprintf(fid,'1 0. 0.	1. 0. 0. 0.		! Heave\n');
            case 4
                fprintf(fid,'2 1. 0. 0. %f %f %f		! Roll about a point\n',RefP);
            case 5
                fprintf(fid,'2 0. 1. 0. %f %f %f		! Pitch about a point\n',RefP);
            case 6
                fprintf(fid,'2 0. 0. 1. %f %f %f		! Yaw about a point\n',RefP);
        end
    end

    fprintf(fid,'%d                          ! Number of resulting generalised forces\n',nDoF);

    for iDoF = 1:nDoF
        switch DoF(iDoF)
            case 1
                fprintf(fid,'1 1. 0.	0. 0. 0. 0.		! Force in x direction\n');
            case 2
                fprintf(fid,'1 0. 1.    0. 0. 0. 0.		! Force in y direction\n');
            case 3
                fprintf(fid,'1 0. 0.	1. 0. 0. 0.		! Force in z direction\n');
            case 4
                fprintf(fid,'2 1. 0. 0. %f %f %f		! Moment force in x direction about a point\n',RefP);
            case 5
                fprintf(fid,'2 0. 1. 0. %f %f %f		! Moment force in y direction about a point\n',RefP);
            case 6
                fprintf(fid,'2 0. 0. 1. %f %f %f		! Moment force in z direction about a point\n',RefP);
        end
    end
    fprintf(fid,'0                      ! Number of lines of additional information \n');
end

fprintf(fid,'--- Load cases to be solved -------------------------------------------------------------------------------------------------------\n');
fprintf(fid,'%g %f %f               ! Number of wave frequencies, Min, and Max (rad/s)\n',length(w),w(1),w(end));
fprintf(fid,'%g %f %f               ! Number of wave directions, Min and Max (degrees)\n',length(dir),dir(1),dir(end));
fprintf(fid,'--- Post processing ---------------------------------------------------------------------------------------------------------------\n');
fprintf(fid,'1	0.1	10.             ! IRF 				! IRF calculation (0 for no calculation), time step and duration\n');
fprintf(fid,'0                      ! Show pressure\n');
fprintf(fid,'0	0.	180.        	! Kochin function 		! Number of directions of calculation (0 for no calculations), Min and Max (degrees)\n');
fprintf(fid,'0	50	400.	400.	! Free surface elevation 	! Number of points in x direction (0 for no calcutions) and y direction and dimensions of domain in x and y direction\n');%%%%%%%%%%%%%%%%%%%%%%
fprintf(fid,'---');
status = fclose(fid);
fclose('all');





%% Run Nemoh

fid = fopen([FileName,filesep,'input.txt'],'wt');
fprintf(fid,' \n 0 \n');
status = fclose(fid);

% Calcul des coefficients hydrodynamiques
OS = isunix;
if OS == 1
    fprintf('\n------ Starting NEMOH ----------- \n');
    system('preProc');
    fprintf('------ Solving BVPs ------------- \n');
    system('Solver');
    fprintf('------ Postprocessing results --- \n');
    system('postProc');
else
    fprintf('\n------ Starting NEMOH ----------- \n');
    system('.\Nemoh\preProcessor.exe');
    fprintf('------ Solving BVPs ------------- \n');
    system('.\Nemoh\Solver.exe');
    fprintf('------ Postprocessing results --- \n');
    system('.\Nemoh\postProcessor.exe');
end



%% Read results



%% Excitation force
nw  = length(w);

fid   = fopen([FileName,filesep,'results',filesep,'ExcitationForce.tec'],'r');
ligne = fgetl(fid);
for c = 1:nDoF*nBi
    ligne = fgetl(fid);
end
ligne = fgetl(fid);

Famp = zeros(nw,nDoF*nBi);
Fphi = zeros(nw,nDoF*nBi);
for k = 1:nw
    ligne = fscanf(fid,'%f',1+(2*nDoF)*nBi);
    w(ii) = ligne(1);
    for jj = 1:nDoF*nBi
        Famp(k,jj) = ligne(2*jj);
        Fphi(k,jj) = ligne(2*jj+1);
    end
end
Fphi = - Fphi; %%%% Nemoh has the "wrong" phase convention!!!!!!
Fe = Famp.*exp(1i*Fphi);
status = fclose(fid);

%% Froude-Krylov force
fid   = fopen([FileName,filesep,'results',filesep,'FKForce.tec'],'r');
ligne = fgetl(fid);
for c = 1:nDoF*nBi
    ligne = fgetl(fid);
end
ligne = fgetl(fid);

Famp = zeros(nw,nDoF*nBi);
Fphi = zeros(nw,nDoF*nBi);
for k = 1:nw
    ligne = fscanf(fid,'%f',1+(2*nDoF)*nBi);
    w(ii) = ligne(1);
    for jj = 1:nDoF*nBi
        Famp(k,jj) = ligne(2*jj);
        Fphi(k,jj) = ligne(2*jj+1);
    end
end
Fphi = - Fphi; %%%% Nemoh has the "wrong" phase convention!!!!!!
Ffk = Famp.*exp(1i*Fphi);
status = fclose(fid);

%% Diffraction force
fid   = fopen([FileName,filesep,'results',filesep,'DiffractionForce.tec'],'r');
ligne = fgetl(fid);
for c = 1:nDoF*nBi
    ligne = fgetl(fid);
end
ligne = fgetl(fid);

Famp = zeros(nw,nDoF*nBi);
Fphi = zeros(nw,nDoF*nBi);
for k = 1:nw
    ligne = fscanf(fid,'%f',1+(2*nDoF)*nBi);
    w(ii) = ligne(1);
    for jj = 1:nDoF*nBi
        Famp(k,jj) = ligne(2*jj);
        Fphi(k,jj) = ligne(2*jj+1);
    end
end
Fphi = - Fphi; %%%% Nemoh has the "wrong" phase convention!!!!!!
Fd=Famp.*exp(1i*Fphi);
status = fclose(fid);

%%

fid    = fopen([FileName,filesep,'results',filesep,'RadiationCoefficients.tec'],'r');
ligne  = fgetl(fid);
for ii=1:nDoF*nBi
    ligne = fgetl(fid);
end

A = zeros(nBi*nDoF,nBi*nDoF,nw);
B = zeros(nBi*nDoF,nBi*nDoF,nw);
for ii = 1:nBi*nDoF
    ligne = fgetl(fid);
    for k = 1:nw
        ligne = fscanf(fid,'%f',1+(2*nDoF)*nBi);
        for jj = 1:nDoF*nBi
            A(ii,jj,k) = ligne(2*jj);
            B(ii,jj,k) = ligne(2*jj+1);
        end
        ligne = fgetl(fid);
    end
end
status = fclose(fid);


%%
Fe  = transpose(Fe);
Fd  = transpose(Fd);
Ffk = transpose(Ffk);

end