% Script written by Dr. Giuseppe Giorgi (ORCID: 0000-0002-1649-9438) under the H2020-MCSA-IF-2018 fellowship
% Grant agreement: 832140
% NLFK4ALL - OpenWave: Validation and Optimization of an Open-Source Novel Nonlinear Froude-Krylov Model for Advanced Design of Wave Energy Converters
% Last update: 09/11/2019

addpath(genpath([cd,filesep,'SupportingFunctions']))
setDefaults
clearvars
close all
clc
%%
% This script purports to use a Boundary Element Method (BEM) sofwtare to verify the correctness of the implementation.
% If the nonlinear Froude-Krylov (NLFK) approach is applied in linear conditions (zero-displacements and very small waves),
% then it's results must agree with the linear Froude-Krylov (LFK) approach. 

%%
flags.runNemoh       = 1;
flags.computeNLFK    = 1;
flags.plotComparison = 1;

%%
% geomName = 'Cyl';
% geomName = 'Cone';
% geomName = 'Hollow_Cyl';
% geomName = 'Hollow_Cone';
% geomName = 'Buoy';
% geomName = 'Buoy';
geomName = 'Buoy';


folderName = 'Floaters';
if ~exist([folderName,filesep,geomName,'.mat'],'file')
    fprintf('The test case %s doesn''t exist. Please launch RUN_defineNLFKgeom first.\n',geomName)
    return
end
load([folderName,filesep,geomName,'.mat'],'NLFKgeom','rhoWater','g');

%% Run Nemoh

if flags.runNemoh
    
    DoF  = [1 3 5] ;       % Array of degrees of freedom: 1: surge, 2: sway, 3: heave, 4: roll, 5: pitch, 6:yaw
    nDoF = length(DoF);

    RefP  = [0 0 NLFKgeom.zRefP] ; % Coordinates of the reference point [xR yR zR] around which torques are computed
    Npan  = 500 ;                  % Target number of panels of the mesh
    n_ang = 35 ;                   % Number of points for angular discretisation

    % For the computation of the hydrostatic coefficient Nemoh assumes CoG==RefP!!!

    x = NLFKgeom.xz(:,1)';
    z = NLFKgeom.xz(:,2)';
    inputGeom(1).name  = geomName;
    inputGeom(1).r     = x(x>=0);
    inputGeom(1).z     = z(x>=0);
    inputGeom(1).n_ang = n_ang;
    inputGeom(1).DoF   = DoF;
    inputGeom(1).RefP  = RefP;
    inputGeom(1).Npan  = Npan;

    nBi = length(inputGeom);                % Number of bodies, just one in this case
    %% Launch Nemoh

    D       = 2*max(NLFKgeom.xz(:,1)); % Characteristic dimension of the device
    Lambdas = D*[0.25 250];             % Minimum and maximum wave lenghts to analyze, roughly
    
    W0 = sqrt(g*2*pi/Lambdas(end));    % Min wave frequency [rad/s]
    W  = sqrt(g*2*pi/Lambdas(1));      % Max wave frequency [rad/s]
    nW = 80;                           % Number of wave frequencies [rad/s]
    w  = linspace(W0,W,nW)';

    dir   = 0;                         % Wave direction
    depth = 100;                       % Water depth (0 if infinite water depth)

    cd NEMOH
    [A, B, Fe, Ffk, Fd, Hydrostatic_nemoh] = FUN_NemohAxi(geomName, inputGeom, w, dir, depth, rhoWater) ; % Run Nemoh
    Hydrostatic_nemoh.inputGeom            = inputGeom;
    cd ..
    close all
    %% Convert from 3- to 6-degree of freedoms
    
    A_   = zeros(6,6,length(w));
    B_   = zeros(6,6,length(w));
    Fe_  = zeros(6,length(w));
    Fd_  = zeros(6,length(w));
    Ffk_ = zeros(6,length(w));
    
    A_  (DoF,DoF,:) = A;
    B_  (DoF,DoF,:) = B;
    Fe_ (DoF,:)     = Fe;
    Fd_ (DoF,:)     = Fd;
    Ffk_(DoF,:)     = Ffk;
    
    A   = A_;
    B   = B_;
    Fe  = Fe_;
    Fd  = Fd_;
    Ffk = Ffk_;
    
    save([folderName,filesep,geomName,'.mat'],'w', 'A', 'B', 'Fe', 'Ffk', 'Fd', 'Hydrostatic_nemoh', 'depth', '-append');
else
    load([folderName,filesep,geomName,'.mat'],'w', 'Ffk', 'depth');    
end
%% Computing the nonlinear dynamic Froude-Krylov forces (FKdy), but in very linear conditions (no displacement and small wave)

if flags.computeNLFK

    etaAmp = 1e-3*abs(diff((NLFKgeom.xz(1:2,2)))); % Based on the geometry, I am assumed a small value for the wave amplitude
    states = [0 0 0 0 0 0]';                       % at rest
    
    par.rhoWater = rhoWater;            
    par.g        = g;                   
    par.gamma    = par.g*par.rhoWater; 
    par.Mass     = par.rhoWater * NLFKgeom.VSub;   % Mass of the floater, assumed to be the same as the mass of the displaced volume
    
    % Relative and absolute tollerances for the numerical integration
    par.RelTol = 1e-4; % default 1e-6
    par.AbsTol = 1e-6; % default 1e-10
    
    % Model for intersection with the free surface
    par.modIntersect = 'flat';
    
    %% Run each regular wave over one period and compute the consequent NLFK
    NLFKdy = zeros(6,length(w)); % Initialize the vector for the nonlinear Froude-Krylov force
    
    waitHandle = waitbar(0,'Please wait...');
    for iw = 1:length(w)
        waitbar(iw/length(w),waitHandle)
        
        % Compute the wave number
        k = w(iw)^2/g; % Infinite water-depth conditions 
        if (depth==0) || 2*pi/k < 2*depth 
            % Function for defining eta=free surface elevation
            par.etaFun = @(x, t)                  etaAmp*cos(w(iw)*t - k*x);  
            % Dynamic pressure formulation with the inclusion of Wheeler stretching
            par.pDy    = @(x, z, t, xR) par.gamma*etaAmp*cos(w(iw)*t - k*x).*exp(k*(z - par.etaFun(xR, t)));  
        else
            k  = fzero(@(k) w(iw)^2 - k*g*tanh(depth*k), w(iw)^2/g);  
            % Function for defining eta=free surface elevation
            par.etaFun = @(x, t)                  etaAmp*cos(w(iw)*t - k*x);  
            % Dynamic pressure formulation with the inclusion of Wheeler stretching
            par.pDy    = @(x, z, t, xR) par.gamma*etaAmp*cos(w(iw)*t - k*x).*cosh(k*depth/(depth + par.etaFun(xR, t))*(z + depth))/cosh(k*depth);    
        end                                          
        

        Tw       = 2*pi/w(iw);         % Wave period
        t        = linspace(0,Tw,200); % Time vector
        NLFKdy_t = zeros(6,length(t));
        
        for it = 1:length(t)
            [~, NLFKdy_t(:,it)] = FUN_NLFK_st_dy(par, NLFKgeom, states, t(it));
        end

        for iDof = [1 3 5]
            [Max, iMax]     = max(NLFKdy_t(iDof,:)); % Find the max of the NLFK time series
            Ph              = -2*pi*t(iMax)/Tw;      % Compute the phase with respect to the wave
            NLFKdy(iDof,iw) = Max/etaAmp*exp(1i*Ph); % Compute the consequent complex NLFK
        end
    end
    close(waitHandle)
    
    %%
    save([folderName,filesep,geomName,'.mat'], 'NLFKdy', '-append');
else
    load([folderName,filesep,geomName,'.mat'],'w', 'NLFKdy');
end



%% Plot comparison
if flags.plotComparison
    for ii = [1 3 5]
        figure(ii)
        subplot(2,1,1)
        plot(w,abs(Ffk(ii,:)),'.',w,abs(NLFKdy(ii,:)),':')
        xlabel('$\omega [rad/s]$')
        if ii<=3
            ylabel('$|F_{FK}| [N]$')
        else
            ylabel('$|F_{FK}| [Nm]$')
        end
        legend('Nemoh','NLFK')
        
        subplot(2,1,2)
        plot(w,angle(Ffk(ii,:))*180/pi,w,angle(NLFKdy(ii,:))*180/pi,':')
        xlabel('$\omega [rad/s]$')
        ylabel('$\angle F_{FK} [^\circ]$')
            
    end
    spreadfigures([figure(1) figure(3) figure(5)],'horizontal')
end


rmpath(genpath([cd,filesep,'SupportingFunctions']))