% Script written by Dr. Giuseppe Giorgi (ORCID: 0000-0002-1649-9438) under the H2020-MCSA-IF-2018 fellowship
% Grant agreement: 832140
% NLFK4ALL - OpenWave: Validation and Optimization of an Open-Source Novel Nonlinear Froude-Krylov Model for Advanced Design of Wave Energy Converters
% Last update: 09/11/2019

addpath(genpath([cd,filesep,'SupportingFunctions']))
setDefaults
clearvars
close all
clc

%% Notation and conventions
% Axisymmetric floating geometry, composed of cylinders and cones
% WF  = World-frame, right-handed, z positive upwards, with origin at the still water level
% BF  = Body-frame, right-handed, z positive upwards and conciding with the WF z-axis, with origin at the reference point (RefP)
% CoG = Center of Gravity, on the axis of the axisymmetric floater
% RefP = Reference point, fixed on the body, on the axis of the axisymmetric floater
% states = [xR, yR, zR, PHI, THETA, PSI] = 6-degree of freedom displacements of the RefP in the WF
% Angles according to the 3-2-1 Euler rotation convention

%% Flags
% 1 if yes, 0 if no:
flags.defineNLFKgeometry          = 1;
flags.plotFloater                 = 1; 
flags.plotEta                     = 1;
flags.computeGeometricProperties  = 1;
flags.computeHydrostaticStiffness = 1;

%%
% See help of FUN_defineNLFKgeom for details on the definition of zCoG and xz

% geomName = 'Cyl';
% geomName = 'Cone';
% geomName = 'Hollow_Cyl';
% geomName = 'Hollow_Cone';
geomName = 'Buoy';

folderName = 'Floaters';
if ~exist(folderName,'dir')
    mkdir(folderName)
end
    
if flags.defineNLFKgeometry
    zCoG  = -3;               
    zRefP = 0; % Common choices are zRefP = 0 or zRefP = zCoG
    if strcmp(geomName,'Cyl')
        xz   = [2 1; 2 -5; 0 -5];
    elseif strcmp(geomName,'Cone')
        xz   = [2.5 1; 1.5 -5; 0 -5];
    elseif strcmp(geomName,'Hollow_Cyl')
        xz   = [2 1; 2 -5; 1 -5; 1 1];
    elseif strcmp(geomName,'Hollow_Cone')
        xz   = [2.5 1; 1.5 -5; 0.5 -5; 0.5 1];
    elseif strcmp(geomName,'Buoy')
        xz   = [3 1.5; 3 -2; 1.75 -3.25; 1.75 -7; 0 -7];
    else
        xz   = []; % Please define the geometry here
    end

    NLFKgeom = FUN_defineNLFKgeom(zCoG, zRefP, xz);
    if exist([folderName,filesep,geomName,'.mat'],'file')
       reply = input(sprintf('%s already exists. Do you want to override it? y/n [y]:',[folderName,filesep,geomName,'.mat']),'s');
       if isempty(reply)
          reply = 'y';
       end
       if strcmp(reply,'y')
           save([folderName,filesep,geomName,'.mat'],'NLFKgeom');
       else
           disp('Please change geomName in order to avoid data loss.')
           return
       end
    else
        save([folderName,filesep,geomName,'.mat'],'NLFKgeom');
    end
else
    load([folderName,filesep,geomName,'.mat'],'NLFKgeom');
end

%% Plotting of the floater (end eventually the surface elevation) in the world-frame and body-frame

if flags.plotFloater    
    xR    = (0.5-rand)*2  *max(NLFKgeom.xz(:,1));   % x displacement of the RefP, in the WF
    yR    = (0.5-rand)*2  *max(NLFKgeom.xz(:,1));   % y displacement of the RefP, in the WF
    zR    = (0.5-rand)*0.5*max(NLFKgeom.xz(:,2));   % z displacement of the RefP, in the WF
    PHI   = (0.5-rand)*pi/4;                        % rotation about an axis parallel to the x-axis, passing through the RefP
    THETA = (0.5-rand)*pi/4;                        % rotation about an axis parallel to the y-axis, passing through the RefP
    PSI   = (0.5-rand)*pi/4;                        % rotation about an axis parallel to the z-axis, passing through the RefP
    
    states = [xR, yR, zR, PHI, THETA, PSI]';

    if flags.plotEta
        g      = 9.806;                          % Acceleration of gravity
        etaAmp = 0.5;                              % Wave amplitude
        Tw     = 6;                              % Wave period
        ww     = 2*pi/Tw;                        % Wave frequency
        t      = rand*Tw;                        % Time instant
        k      = ww^2/g;                         % Wave number
        etaFun = @(x, t) etaAmp*cos(ww*t - k*x); % Function for defining eta=free surface elevation
    else
        t      = nan;
        etaFun = nan;
    end
    VIEW = [0 0]; % define aximuth and elevation angles of the 3D view
    
    FUN_plot3D(NLFKgeom, states, VIEW, etaFun, t)
end

%% Compute and check lateral surface and volume, total and submerged and centre of buoyancy
if flags.computeGeometricProperties
    NLFKgeom = FUN_computeBodyProperties(NLFKgeom);    
    save([folderName,filesep,geomName,'.mat'],'NLFKgeom','-append');
end

%% Compute the hydrostatic stiffness matrix Kh
if flags.computeHydrostaticStiffness
    rhoWater     = 1025;                % Water density
    g            = 9.806;               % Acceleration of gravity
    par.rhoWater = rhoWater;            
    par.g        = g;                   
    par.gamma    = par.g*par.rhoWater; 
    par.Mass     = par.rhoWater * NLFKgeom.VSub;   % Mass of the floater, assumed to be the same as the mass of the displaced volume

    % Define the free surface elevation and dynamic pressure (both zero for computing Kh)
    par.etaFun = @(x, t) zeros(size(x));
    par.pDy    = @(x, z, t, xR) zeros(size(x));   
    
    % Relative and absolute tollerances for the numerical integration
    par.RelTol = 1e-6; % default 1e-6
    par.AbsTol = 1e-10; % default 1e-10
    
    % Model for intersection with the free surface
    par.modIntersect = 'linear';
    
    t   = 0;    % Time instant (not needed for computing Kh)
    dz  = 1e-3; % Infinitesimal increment in the heave direction
    dth = 1e-4; % Infinitesimal increment in the roll and pitch directions
    
    [FKst_0,~] = FUN_NLFK_st_dy(par, NLFKgeom, [0 0 0  0   0   0] , t); % Static FK at rest
    [FKst_3,~] = FUN_NLFK_st_dy(par, NLFKgeom, [0 0 dz 0   0   0] , t); % Static FK for infinitesimal displacement in heave
    [FKst_4,~] = FUN_NLFK_st_dy(par, NLFKgeom, [0 0 0  dth 0   0] , t); % Static FK for infinitesimal displacement in roll
    [FKst_5,~] = FUN_NLFK_st_dy(par, NLFKgeom, [0 0 0  0   dth 0] , t); % Static FK for infinitesimal displacement in pitch


    Kh      = zeros(6,6);
    Kh(3,3) = -(FKst_3(3) - FKst_0(3))/dz;
    Kh(4,4) = -(FKst_4(4) - FKst_0(4))/dth;
    Kh(5,5) = -(FKst_5(5) - FKst_0(5))/dth;
    
   %% Algebraic computation of Kh, used to check that the nonlinear approach
   
    Iarea = pi/4*(NLFKgeom.Rout_SWL^4 - NLFKgeom.Rin_SWL^4);
    BG    = NLFKgeom.zCoG - NLFKgeom.zB;

    Kh_alg      = zeros(6,6);
    Kh_alg(3,3) = par.gamma*pi*(NLFKgeom.Rout_SWL^2 - NLFKgeom.Rin_SWL^2);
    Kh_alg(4,4) = par.gamma*(Iarea - NLFKgeom.VSub*BG);
    Kh_alg(5,5) = par.gamma*(Iarea - NLFKgeom.VSub*BG);

    if abs((Kh(3,3)-Kh_alg(3,3))/Kh_alg(3,3))<=1e-3 && abs((Kh(4,4)-Kh_alg(4,4))/Kh_alg(4,4))<=1e-3 && abs((Kh(5,5)-Kh_alg(5,5))/Kh_alg(5,5))<=1e-3
        disp('CORRECT! Stiffness matrix:')
        disp(num2str(Kh))
    else
        disp('PROBLEMS for the stiffness matrix integral!!!!!!!')
        disp('Computed: ')
        disp(num2str(Kh))
        disp('Algebraic: ')
        disp(num2str(Kh_alg))
    end

    
    save([folderName,filesep,geomName,'.mat'],'Kh','NLFKgeom','rhoWater','g','-append');
    
end



rmpath(genpath([cd,filesep,'SupportingFunctions']))